@@ -16,7 +16,9 @@ const (
 )
 
 var (
-	counters descMap
+	counters            descMap
+	serviceTimes        descMap // ExtractServiceTimes decides if we want to extract service times
+	ExtractServiceTimes bool
 )
 
 /*Exporter entry point to squid exporter */
@@ -30,18 +32,35 @@ type Exporter struct {
 	up     *prometheus.GaugeVec
 }
 
+type CollectorConfig struct {
+	Hostname string
+	Port     int
+	Login    string
+	Password string
+	Labels   config.Labels
+	Headers  []string
+}
+
 /*New initializes a new exporter */
-func New(hostname string, port int, login string, password string, labels config.Labels) *Exporter {
-	counters = generateSquidCounters(labels.Keys)
-	c := NewCacheObjectClient(hostname, port, login, password)
+func New(c *CollectorConfig) *Exporter {
+	counters = generateSquidCounters(c.Labels.Keys)
+	if ExtractServiceTimes {
+		serviceTimes = generateSquidServiceTimes(c.Labels.Keys)
+	}
 
 	return &Exporter{
-		c,
+		NewCacheObjectClient(&CacheObjectRequest{
+			c.Hostname,
+			c.Port,
+			c.Login,
+			c.Password,
+			c.Headers,
+		}),
 
-		hostname,
-		port,
+		c.Hostname,
+		c.Port,
 
-		labels,
+		c.Labels,
 		prometheus.NewGaugeVec(prometheus.GaugeOpts{
 			Namespace: namespace,
 			Name:      "up",
@@ -59,6 +78,12 @@ func (e *Exporter) Describe(ch chan<- *prometheus.Desc) {
 		ch <- v
 	}
 
+	if ExtractServiceTimes {
+		for _, v := range serviceTimes {
+			ch <- v
+		}
+	}
+
 }
 
 /*Collect fetches metrics from squid manager and pushes them to promethus */
@@ -74,7 +99,22 @@ func (e *Exporter) Collect(c chan<- prometheus.Metric) {
 		}
 	} else {
 		e.up.With(prometheus.Labels{"host": e.hostname}).Set(0)
-		log.Println("Could not fetch metrics from squid instance: ", err)
+		log.Println("Could not fetch counter metrics from squid instance: ", err)
 	}
+
+	if ExtractServiceTimes {
+		insts, err = e.client.GetServiceTimes()
+
+		if err == nil {
+			for i := range insts {
+				if d, ok := serviceTimes[insts[i].Key]; ok {
+					c <- prometheus.MustNewConstMetric(d, prometheus.CounterValue, insts[i].Value, e.labels.Values...)
+				}
+			}
+		} else {
+			log.Println("Could not fetch service times metrics from squid instance: ", err)
+		}
+	}
+
 	e.up.Collect(c)
 }
